/////////////////////////////////////////////////////////////////////////////
// Connection.cpp : implementation file
//

#include "stdafx.h"

#include "FireWallDoc.h"
#include "ConnectSocket.h"
#include "LocalSocket.h"
#include "GlobalSocket.h"
#include "Connection.h"
#include "Base64.h"

#define BUFF_LEN 4096
#define STR_LEN 256
#define HTTP_PORT 80

/////////////////////////////////////////////////////////////////////////////
// CConnection

CConnection::CConnection(CFireWallDoc *pDoc)
{
	m_pDoc = pDoc;
	m_pLocal = NULL;
	m_pGlobal = NULL;
	m_isClosing = FALSE;
}

CConnection::~CConnection()
{
	if (m_pLocal != NULL)
		delete m_pLocal;
	if (m_pGlobal != NULL)
		delete m_pGlobal;
}

/////////////////////////////////////////////////////////////////////////////
// CConnection member functions

BOOL CConnection::AttachLocal(CSocket &local)
{
	char msg[STR_LEN];
	CString ip;
	UINT port;

	if (m_pLocal!=NULL)
		delete m_pLocal;

	m_pLocal = new CLocalSocket(this);

	if (!local.Accept(*m_pLocal))
	{
		delete m_pLocal;
		m_pLocal = NULL;
		return FALSE;
	}
	m_pLocal->Init();

	m_pLocal->GetPeerName(ip, port);
	sprintf(msg, "Connected locally to %s:%u", ip, port);
	m_pDoc->Message(msg);

	return TRUE;
}

BOOL CConnection::AttachGlobal(CSocket &global)
{
	char msg[STR_LEN];
	CString ip;
	UINT port;
	SOCKET sock;

	if (m_pGlobal!=NULL)
		delete m_pGlobal;

	m_pGlobal = new CGlobalSocket(this);

	if(!m_pGlobal->Attach(sock = global.Detach()))
	{
		global.Attach(sock);
		delete m_pGlobal;
		m_pGlobal = NULL;
		return FALSE;
	}
	m_pGlobal->Init();
	
	m_pGlobal->GetPeerName(ip, port);
	sprintf(msg, "Connected globally to %s:%u", ip, port);
	m_pDoc->Message(msg);

	return TRUE;
}

void CConnection::Close()
{
	if (!m_isClosing)
	{
		m_isClosing = TRUE;
		m_pDoc->CloseConnection(this);
	}
}

void CConnection::GetLocalMessage()
{
	char msg[BUFF_LEN];
	UINT msgLen;
	char server[STR_LEN] = "";
	CSocket socket;

	msgLen = m_pLocal->ReceiveMessage(msg, sizeof(msg));

/* Display client request
	msg[msgLen] = '\0';
	m_pDoc->Message(msg);
*/

	ProcessRequest(msg, server);

	socket.Create();
	if (!socket.Connect(server, HTTP_PORT))
	{
		m_pDoc->Message("Connection to server failed.");
		return;
	}
	if (!AttachGlobal(socket))
	{
		m_pDoc->Message("Connection to server failed.");
		return;
	}
	m_pGlobal->SendMessage(msg, msgLen);
}

void CConnection::GetGlobalMessage()
{
	char msg[BUFF_LEN];
	UINT msgLen;

	msgLen = m_pGlobal->ReceiveMessage(msg, sizeof(msg));

	m_pLocal->SendMessage(msg, msgLen);
}

void CConnection::ProcessRequest(char *msg, char *server)
{
	char get[] = "GET http://";
	char *p1, *p2;
	int n;

	p1 = strstr(msg, get) + strlen(get);
	p2 = strstr(p1, "/");
	n = p2 - p1;

	memcpy(server, p1, n);
	server[n] = '\0';

	memset(p1 - 7, ' ', n + 7);
}
